#ifndef __DirectoryData_Act_h__
#define __DirectoryData_Act_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Exception.h>

#include <Base4c/Guid.h>

#include <librfcdal/DalDatabase.h>

namespace DirectoryData
{
	class DirectoryData_Export Act : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Base4c::Guid _guidAct;

				public:
					Key();
					Key( const Base4c::Guid& guidAct );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Guid& act() const;

					std::string str() const;
			};

		private:
			Key _key;
			std::string _strTitle;
			std::string _strNode;

		public:
			Act();
			Act( const Key& key );
			Act( const Act& act );
			virtual ~Act();

			Act& operator=( const Act& act );
			Act& merge( const Act& act );

			const Key& key() const;
			const std::string& title() const;
			const std::string& node() const;

			void setTitle( const std::string& strTitle );
			void setNode( const std::string& strNode );

			static Act fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Act create( const Act& act ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export ActMap : public std::map<Act::Key, Act>
	{
		public:
			static ActMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Act_h__ */
