#ifndef __DirectoryData_Address_h__
#define __DirectoryData_Address_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Exception.h>
#include <DirectoryData/AddressTemplate.h>
#include <DirectoryData/Base.h>

#include <ISOData/Country.h>
#include <ISOData/Subdivision.h>

#include <librfcdal/SqlGSPoint.h>
#include <librfcdal/DalDatabase.h>

#include <Base4c/Guid.h>

#include <string>
#include <map>

namespace DirectoryData
{
	class MerchantPtrMap;
	class PitchPtrMap;

	class DirectoryData_Export Address : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Base4c::Guid _guidAddress;

				public:
					Key();
					Key( const Base4c::Guid& guidAddress );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Guid& address() const;

					std::string str() const;
			};

		private:
			Key _key;
			std::string _strUnitNo;
			std::string _strBuilding;
			std::string _strStreetNo;
			std::string _strStreet;
			std::string _strDistrict;
			std::string _strTown;
			std::string _strRegion;
			std::string _strPostCode;
			const ISOData::Country* _pcountry;
			const ISOData::Subdivision* _psubdivision;
			const AddressTemplate* _paddressTemplate;
			RfcDal::SqlGSPoint _location;
			MerchantPtrMap* _pmerchantS;
			PitchPtrMap* _ppitchS;

		public:
			Address();
			Address( const Address::Key& key );
			Address( const Address& address );
			virtual ~Address();

			Address& operator=( const Address& address );
			virtual Address& merge( const Address& address );

			const Key& key() const;
			const std::string& unitNo() const;
			const std::string& building() const;
			const std::string& streetNo() const;
			const std::string& street() const;
			const std::string& district() const;
			const std::string& town() const;
			const std::string& region() const;
			const std::string& postCode() const;
			const ISOData::Country& isoCountry() const throw( DirectoryData::Exception );
			const ISOData::Subdivision& isoSubdivision() const throw( DirectoryData::Exception );
			const AddressTemplate& addressTemplate() const throw( DirectoryData::Exception );
			const RfcDal::SqlGSPoint& location() const;
			const MerchantPtrMap& merchants() const;
			const PitchPtrMap& pitchs() const;

			std::string formatShort() const throw( DirectoryData::Exception );
			std::string formatFull() const throw( DirectoryData::Exception );

			void setUnitNo( const std::string& strUnitNo );
			void setBuilding( const std::string& strBuilding );
			void setStreetNo( const std::string& strStreetNo );
			void setStreet( const std::string& strStreet );
			void setDistrict( const std::string& strDistrict );
			void setTown( const std::string& strTown );
			void setRegion( const std::string& strRegion );
			void setPostCode( const std::string& strPostCode );
			void setISOCountry( const ISOData::Country& country );
			void setISOSubdivision( const ISOData::Subdivision& subdivision );
			void setAddressTemplate( const AddressTemplate& addressTemplate );
			void setLocation( const RfcDal::SqlGSPoint& location );

			MerchantPtrMap& merchants();
			PitchPtrMap& pitchs();

			static Address fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Address create( const Address& address ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export AddressMap : public std::map<Address::Key, Address>
	{
		public:
			static AddressMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Address_h__ */
