#ifndef __DirectoryData_AddressTemplate_h__
#define __DirectoryData_AddressTemplate_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Exception.h>
#include <DirectoryData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace DirectoryData
{
	class DirectoryData_Export AddressTemplate : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					std::string _strAddressTemplate;

				public:
					Key();
					Key( const std::string& strAddressTemplate );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& addressTemplate() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strTemplateS;
			std::string _strTemplateL;

		public:
			AddressTemplate();
			AddressTemplate( const AddressTemplate::Key& key );
			AddressTemplate( const AddressTemplate& temp );
			virtual ~AddressTemplate();

			AddressTemplate& operator=( const AddressTemplate& temp );
			virtual AddressTemplate& merge( const AddressTemplate& temp );

			const Key& key() const;
			const std::string& templateS() const;
			const std::string& templateL() const;

			void setTemplateS( const std::string& strTemplateS );
			void setTemplateL( const std::string& strTemplateL );

			static AddressTemplate fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static AddressTemplate create( const AddressTemplate& temp ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export AddressTemplateMap : public std::map<AddressTemplate::Key, AddressTemplate>
	{
		public:
			static AddressTemplateMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_AddressTemplate_h__ */
