#ifndef __DirectoryData_BusinessLogin_h__
#define __DirectoryData_BusinessLogin_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Exception.h>
#include <DirectoryData/Merchant.h>

#include <Auth4cData/User.h>

#include <map>

namespace DirectoryData
{
	class DirectoryData_Export BusinessLogin : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Auth4cData::User _user;
					Merchant* _pmerchant;

				public:
					Key();
					Key( const Auth4cData::User& user, Merchant& merchant );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Auth4cData::User& user() const;
					Merchant& merchant() const throw( DirectoryData::Exception );

					virtual std::string str() const throw( DirectoryData::Exception );
			};

		private:
			Key _key;

		public:
			BusinessLogin();
			BusinessLogin( const Key& key );
			BusinessLogin( const BusinessLogin& login );
			virtual ~BusinessLogin();

			BusinessLogin& operator=( const BusinessLogin& login );
			bool operator==( const BusinessLogin& login ) const;

			const Key& key() const;

			static BusinessLogin fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static BusinessLogin create( const BusinessLogin& login ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export BusinessLoginMap : public std::map<BusinessLogin::Key, BusinessLogin>
	{
		public:
			static BusinessLoginMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_BusinessLogin_h__ */
