#ifndef __DirectoryData_Category_h__
#define __DirectoryData_Category_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/MasterCategory.h>
#include <DirectoryData/Exception.h>
#include <DirectoryData/Base.h>

#include <librfcdal/DalDatabase.h>

namespace DirectoryData
{
	class DirectoryData_Export Category : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					std::string _strCategory;

				public:
					Key();
					Key( const std::string& strCategory );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& category() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strDescription;
			const MasterCategory* _pmaster;

		public:
			Category();
			Category( const Category::Key& key );
			Category( const Category& category );
			virtual ~Category();

			Category& operator=( const Category& category );
			virtual Category& merge( const Category& business );

			const Key& key() const;
			const std::string& description() const;
			const MasterCategory& masterCategory() const throw( DirectoryData::Exception );

			void setDescription( const std::string& strDescription );
			void setMasterCategory( const MasterCategory& master );

			static Category fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Category create( const Category& category ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			static Category& refFromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
	};

	class DirectoryData_Export CategoryMap : public std::map<Category::Key, Category>
	{
		public:
			static CategoryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};

	class DirectoryData_Export CategoryPtrMap : public std::map<Category::Key, Category*>
	{
		public:
			static CategoryPtrMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Category_h__ */
