#ifndef __DirectoryData_DirectoryFactory_h__
#define __DirectoryData_DirectoryFactory_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Exception.h>

#include <DirectoryData/MasterCategory.h>
#include <DirectoryData/Category.h>
#include <DirectoryData/AddressTemplate.h>
#include <DirectoryData/Address.h>
#include <DirectoryData/MerchantType.h>
#include <DirectoryData/Incorporation.h>
#include <DirectoryData/Food.h>
#include <DirectoryData/PitchType.h>
#include <DirectoryData/FestivalStatus.h>
#include <DirectoryData/Pitch.h>
#include <DirectoryData/Merchant.h>
#include <DirectoryData/Festival.h>
#include <DirectoryData/Stage.h>
#include <DirectoryData/Act.h>
#include <DirectoryData/Gallery.h>
#include <DirectoryData/MessageStatus.h>
#include <DirectoryData/Sender.h>
#include <DirectoryData/Mailer.h>
#include <DirectoryData/Product.h>
#include <DirectoryData/BusinessLogin.h>

#include <DirectoryData/Busy.h>

#include <Auth4cData/AuthenticatedUser.h>

#include <Base4c/Version.h>

namespace DirectoryData
{
	class DirectoryData_Export DirectoryFactory
	{
		private:
			static DirectoryData::MasterCategoryMap _masterCategoryS;
			static DirectoryData::CategoryMap _categoryS;
			static DirectoryData::AddressTemplateMap _addressTemplateS;
			static DirectoryData::AddressMap _addressS;
			static DirectoryData::MerchantTypeMap _merchantTypeS;
			static DirectoryData::IncorporationMap _incorporationS;
			static DirectoryData::FoodMap _foodS;
			static DirectoryData::PitchTypeMap _pitchTypeS;
			static DirectoryData::FestivalStatusMap _festivalStatusS;
			static DirectoryData::MessageStatusMap _messageStatusS;
			static DirectoryData::SenderMap _senderS;
			static DirectoryData::PitchMap _pitchS;
			static DirectoryData::MerchantMap _merchantS;
			static DirectoryData::FestivalMap _festivalS;
			static DirectoryData::Festival* _pfestivalActive;
			static DirectoryData::StageMap _stageS;
			static DirectoryData::ActMap _actS;
			static DirectoryData::GalleryMap _galleryS;

			static DirectoryData::ProductMap _productS;
			static DirectoryData::BusinessLoginMap _loginS;

			static DirectoryData::Mailer _mailer;
			static MessageMap _messageS;

			static DirectoryData::BusyReflector* _pbusyReflector;

		public:
			static const Base4c::Version version();

			static void setBusyReflector( DirectoryData::BusyReflector* preflector );

			static void initialize( const Auth4cData::AuthenticatedUser& user ) throw( DirectoryData::Exception );
			static void initialize( const std::string& strInit ) throw( DirectoryData::Exception );
			static void initMailer() throw( DirectoryData::Exception );

			static DirectoryData::MasterCategoryMap& masterCategorys();
			static DirectoryData::CategoryMap& categorys();
			static DirectoryData::AddressTemplateMap& addressTemplates();
			static DirectoryData::AddressMap& addresses();
			static DirectoryData::MerchantTypeMap& merchantTypes();
			static DirectoryData::IncorporationMap& incorporations();
			static DirectoryData::FoodMap& foods();
			static DirectoryData::PitchTypeMap& pitchTypes();
			static DirectoryData::FestivalStatusMap& festivalStatuss();
			static DirectoryData::MessageStatusMap& messageStatuss();
			static DirectoryData::SenderMap& senders();
			static DirectoryData::PitchMap& pitchs();
			static DirectoryData::FestivalMap& festivals();
			static bool isFestivalActive();
			static DirectoryData::Festival& activeFestival() throw( DirectoryData::Exception );
			static void activateFestival( DirectoryData::Festival& festival ) throw( DirectoryData::Exception );
			static DirectoryData::MerchantMap& merchants();
			static DirectoryData::StageMap& stages();
			static DirectoryData::ActMap& acts();
			static DirectoryData::GalleryMap& gallerys();

			static DirectoryData::ProductMap& products();
			static DirectoryData::BusinessLoginMap& logins();

			static DirectoryData::Mailer& mailer();
			static MessagePtrMap messages( const MessageStatus::Key& key ) throw( DirectoryData::Exception );

		protected:
			static void loadStatic( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadMasterCategorys( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadCategorys( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadAddressTemplates( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadAddresses( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadPitches( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadFestivals( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadMerchants( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadActs( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadStageBookings( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadGallerys( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );

			static void loadProducts( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static void loadLogins( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_DirectoryFactory_h__ */
