#ifndef __DirectoryData_Festival_h__
#define __DirectoryData_Festival_h__

#include <DirectoryData/Base.h>

#include <DirectoryData/FestivalStatus.h>

#include <Base4c/Date.h>

#include <map>

namespace DirectoryData
{
	class FestivalBooking;
	class FestivalBookingPtrMap;

	class DirectoryData_Export Festival : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Base4c::Date _dtFestival;

				public:
					Key();
					Key( const Base4c::Date& dtFestival );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Date& festival() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			bool _bActive;
			const FestivalStatus* _pfestivalStatus;
			FestivalBookingPtrMap* _pfestivalBookingS;

		public:
			Festival();
			Festival( const Festival::Key& key );
			Festival( const Festival& festival );
			virtual ~Festival();

			Festival& operator=( const Festival& festival );
			Festival& merge( const Festival& festival );

			const Key& key() const;
			bool active() const;
			const FestivalStatus& festivalStatus() const throw( DirectoryData::Exception );
			const FestivalBookingPtrMap& festivalBookings() const;

			void setActive( bool bActive = true );
			void setFestivalStatus( const FestivalStatus& status );
			FestivalBookingPtrMap& festivalBookings();
			void addFestivalBooking( FestivalBooking& booking ) throw( DirectoryData::Exception );
			void removeFestivalBooking( const FestivalBooking& booking ) throw( DirectoryData::Exception );

			static Festival fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Festival create( const Festival& festival ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export FestivalMap : public std::map<Festival::Key, Festival>
	{
		public:
			static FestivalMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );

			Festival& festivalBefore( const Festival& festival ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Festival_h__ */
