#ifndef __DirectoryData_FestivalBooking_h__
#define __DirectoryData_FestivalBooking_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Festival.h>
#include <DirectoryData/Merchant.h>
#include <DirectoryData/MerchantType.h>

#include <map>

namespace DirectoryData
{
	class Pitch;
	class PitchBooking;
	class PitchBookingMap;

	class DirectoryData_Export FestivalBooking : public Base
	{
		public:
			class DirectoryData_Export Key : public Festival::Key, public Merchant::Key
			{
				public:
					Key();
					Key( const Festival::Key& kFestival, const Merchant::Key& kMerchant );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					std::string str() const;
			};

		private:
			Key _key;
			const MerchantType* _pMerchantRole;
			int _nPitches;
			int _nSeq;
			int _nSponsorSeq;
			std::string _strComments;
			bool _bRepeats;
			bool _bFood;
			bool _bFoodPermitRcvd;
			std::string _strApplication;
			PitchBookingMap* _ppitchBookingS;
			Festival* _pfestival;
			Merchant* _pmerchant;

		public:
			FestivalBooking();
			FestivalBooking( Festival& festival, Merchant& merchant );
			FestivalBooking( const FestivalBooking& booking );
			virtual ~FestivalBooking();

			FestivalBooking operator=( const FestivalBooking& booking );

			const Key& key() const;
			const MerchantType& merchantRole() const throw( DirectoryData::Exception );
			int pitches() const;
			int seq() const;
			int sponsorSeq() const;
			const std::string& comments() const;
			bool repeats() const;
			bool food() const;
			bool foodPermitRcvd() const;
			const std::string& application() const;
			const PitchBookingMap& pitchBookings() const;

			Festival& festival() const throw( DirectoryData::Exception );
			Merchant& merchant() const throw( DirectoryData::Exception );
			void validate() const throw( DirectoryData::CompoundException );

			void setMerchantRole( const MerchantType& type );
			void setPitches( int nPitches );
			void setSeq( int nSeq );
			void setSponsorSeq( int nSeq );
			void setComments( const std::string& strComments );
			void setRepeats( bool bRepeats );
			void setFood( bool bFood );
			void setFoodPermitRcvd( bool bRcvd );
			void setApplication( const std::string& strApplication );
			PitchBookingMap& pitchBookings();

			PitchBooking& addPitchBooking( Pitch& pitch ) throw( DirectoryData::Exception );
			void removePitchBooking( const PitchBooking& pitchBooking ) throw( DirectoryData::Exception );

			static FestivalBooking fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static FestivalBooking create( const FestivalBooking& booking ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export FestivalBookingMap : public std::map<FestivalBooking::Key, FestivalBooking>
	{
		public:
			static FestivalBookingMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			FestivalBookingMap& operator=( const FestivalBookingMap bookingS );
	};

	class DirectoryData_Export FestivalBookingPtrMap : public std::map<FestivalBooking::Key, FestivalBooking*>
	{
	};
}

#endif /* !__DirectoryData_FestivalBooking_h__ */
