#ifndef __DirectoryData_Gallery_h__
#define __DirectoryData_Gallery_h__

#include <DirectoryData/Festival.h>

namespace DirectoryData
{
	class GalleryImage;
	class GalleryImageMap;

	class DirectoryData_Export Gallery : public Base
	{
		private:
			Festival::Key _key;
			std::string _strPath;
			std::string _strImage;
			std::string _strLegend;
			GalleryImageMap* _pimageS;

		public:
			Gallery();
			Gallery( const Festival::Key& key );
			Gallery( const Gallery& gallery );
			virtual ~Gallery();

			Gallery& operator=( const Gallery& gallery );
			Gallery& merge( const Gallery& gallery );

			const Festival::Key& key() const;
			const std::string& path() const;
			const std::string& image() const;
			const std::string& legend() const;
			const GalleryImageMap& galleryImages() const;

			void setPath( const std::string& strPath );
			void setImage( const std::string& strImage );
			void setLegend( const std::string& strLegend );
			GalleryImageMap& galleryImages();

			GalleryImage& addGalleryImage( const GalleryImage& image ) throw( DirectoryData::Exception );
			void removeGalleryImage( const GalleryImage& image ) throw( DirectoryData::Exception );

			static Gallery fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Gallery create( const Festival& festival ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

			int nextSequence() const throw( DirectoryData::Exception );
			static int nextSequence( const Gallery& gallery ) throw( DirectoryData::Exception );

		protected:
			void loadImages();
	};

	class DirectoryData_Export GalleryMap : public std::map<Festival::Key, Gallery>
	{
		public:
			static GalleryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* __DirectoryData_Gallery_h__ */
