#ifndef __DirectoryData_GalleryImage_h__
#define __DirectoryData_GalleryImage_h__

#include <DirectoryData/Festival.h>

namespace DirectoryData
{
	class Gallery;

	class DirectoryData_Export GalleryImage : public Base
	{
		public:
			class DirectoryData_Export Key : public Festival::Key
			{
				private:
					int _nSeq;

				public:
					Key();
					Key( const Festival::Key& kFestival, int nSeq );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int seq() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strThumbnail;
			std::string _strImage;

		public:
			GalleryImage();
			GalleryImage( const GalleryImage::Key& key );
			GalleryImage( const GalleryImage& image );
			virtual ~GalleryImage();

			GalleryImage& operator=( const GalleryImage& image );
			GalleryImage& merge( const GalleryImage& image );

			const GalleryImage::Key& key() const;
			const std::string& thumbnail() const;
			const std::string& image() const;

			void setThumbnail( const std::string& strThumbnail );
			void setImage( const std::string& strImage );

			Gallery& gallery() const throw( DirectoryData::Exception );

			static GalleryImage suggest( const Gallery& gallery ) throw( DirectoryData::Exception );
			static GalleryImage fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static GalleryImage create( const GalleryImage& image ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export GalleryImageMap : public std::map<GalleryImage::Key, GalleryImage>
	{
		public:
			static GalleryImageMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* __DirectoryData_GalleryImage_h__ */
