#ifndef INVOICEPRODUCT_H
#define INVOICEPRODUCT_H

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/MerchantInvoice.h>
#include <DirectoryData/Product.h>


namespace DirectoryData
{
	class DirectoryData_Export InvoiceProduct : public Base
	{
		public:
			class DirectoryData_Export Key : public MerchantInvoice::Key, public Product::Key
			{
				public:
					Key();
					Key( const MerchantInvoice::Key& keyInvoice, const Product::Key& keyProduct );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			int _nQuantity;
			double _dPrice;

		public:
			InvoiceProduct();
			InvoiceProduct( const Key& key );
			InvoiceProduct( const InvoiceProduct& product );
			virtual ~InvoiceProduct();

			InvoiceProduct& operator=( const InvoiceProduct& product );

			const Key& key() const;
			int quantity() const;
			double price() const;

			void setQuantity( int nQuantity );
			void setPrice( double dPrice );

			static InvoiceProduct fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static InvoiceProduct create( const InvoiceProduct& product ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export InvoiceProductMap : public std::map<InvoiceProduct::Key, InvoiceProduct>
	{
		public:
			static InvoiceProductMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif // INVOICEPRODUCT_H
