#ifndef __DirectoryData_MasterCategory_h__
#define __DirectoryData_MasterCategory_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Exception.h>
#include <DirectoryData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace DirectoryData
{
	class DirectoryData_Export MasterCategory : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					std::string _strMasterCategory;

				public:
					Key();
					Key( const std::string& strMasterCategory );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& masterCategory() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strDescription;

		public:
			MasterCategory();
			MasterCategory( const MasterCategory::Key& key );
			MasterCategory( const MasterCategory& category );
			virtual ~MasterCategory();

			MasterCategory& operator=( const MasterCategory& category );
			virtual MasterCategory& merge( const MasterCategory& category );

			const Key& key() const;
			const std::string& description() const;
			
			void setDescription( const std::string& strDescription );

			static MasterCategory fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static MasterCategory create( const MasterCategory& category ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export MasterCategoryMap : public std::map<MasterCategory::Key, MasterCategory>
	{
		public:
			static MasterCategoryMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_MasterCategory_h__ */
