#ifndef __DirectoryData_Merchant_h__
#define __DirectoryData_Merchant_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Base.h>
#include <DirectoryData/Address.h>
#include <DirectoryData/Category.h>
#include <DirectoryData/MerchantType.h>
#include <DirectoryData/Incorporation.h>
#include <DirectoryData/Food.h>

namespace DirectoryData
{
	class FestivalBooking;
	class FestivalBookingMap;
	class MerchantComment;
	class MerchantCommentMap;
	class PitchPtrMap;

	class DirectoryData_Export Merchant : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					std::string _strMerchant;

				public:
					Key();
					Key( const std::string& strMerchant );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& merchant() const;

					std::string str() const;
			};

		private:
			Key _key;
			bool _bSupplemented;
			const MerchantType* _pmerchantType;
			std::string _strTitle;
			std::string _strLogo;
			std::string _strContact;
			std::string _strAddress;
			Address* _paddress;
			std::string _strTelephone;
			std::string _strEmail;
			std::string _strEmailPublic;
			std::string _strURI;
			std::string _strDescription;
			const Incorporation* _pincorporation;
			std::string _strRegistration;
			const Food* _pfood;
			std::string _strBadgeNo;
			CategoryPtrMap _categoryS;
			bool _bDeleted;
			FestivalBookingMap* _pfestivalBookingS;
			MerchantCommentMap* _pmerchantCommentS;
			PitchPtrMap* _pautoBookPitchS;

		public:
			Merchant();
			Merchant( const Key& key );
			Merchant( const Merchant& merchant );
			virtual ~Merchant();

			Merchant& operator=( const Merchant& merchant );
			virtual Merchant& merge( const Merchant& merchant );

			const Merchant::Key& key() const;
			const MerchantType& merchantType() const throw( DirectoryData::Exception );
			const std::string& title() const;
			const std::string& logo() const;
			const std::string& contact() const;
			const std::string& addressText() const;
			const Address& address() const throw( DirectoryData::Exception );
			const std::string& telephone() const;
			const std::string& email() const;
			const std::string& emailPublic() const;
			const std::string& uri() const;
			const std::string& description() const;
			const Incorporation& incorporation() const throw( DirectoryData::Exception );
			const std::string& registration() const;
			const Food& food() const throw( DirectoryData::Exception );
			const std::string& badgeNo() const;
			const CategoryPtrMap& categorys() const;
			bool deleted() const;
			int autoBook() const;
			const PitchPtrMap& autoBookPitchs() const;
			const FestivalBookingMap& festivalBookings() const;
			const MerchantCommentMap& merchantComments() const;

			void setMerchantType( const MerchantType& merchantType );
			void setTitle( const std::string& strTitle );
			void setLogo( const std::string& strTitle );
			void setContact( const std::string& strContact );
			void setAddress( const std::string& strAddress );
			void setAddress( Address& address );
			void setTelephone( const std::string& strTelephone );
			void setEmail( const std::string& strEmail );
			void setEmailPublic( const std::string& strEmailPublic );
			void setURI( const std::string& strURI );
			void setDescription( const std::string& strDescription );
			void setIncorporation( const Incorporation& incorporation );
			void setRegistration( const std::string& strRegistration );
			void setFood( const Food& food );
			void setBadgeNo( const std::string& strBadgeNo );
			CategoryPtrMap& categorys();
			void setDeleted( bool bDeleted = true );
			PitchPtrMap& autoBookPitchs();

			FestivalBookingMap& festivalBookings();
			FestivalBooking& addFestivalBooking( const FestivalBooking& booking ) throw( DirectoryData::Exception );
			void removeFestivalBooking( FestivalBooking& booking ) throw( DirectoryData::Exception );

			MerchantCommentMap& merchantComments();
			int maxMerchantCommentSeverity() const;
			MerchantComment& addMerchantComment( const MerchantComment& comment ) throw( DirectoryData::Exception );

			Address& address() throw( DirectoryData::Exception );

			static Merchant fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db, bool bSupplement = true ) throw( DirectoryData::Exception );
			void supplement( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			static Merchant create( const Merchant& merchant ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

		protected:
			void updateCategorys( RfcDal::DalDatabase& db ) throw(DirectoryData::Exception );
			void updateAutoBook( RfcDal::DalDatabase& db ) throw(DirectoryData::Exception );
	};

	class DirectoryData_Export MerchantPtrMap : public std::map<Merchant::Key, Merchant*>
	{
	};

	class DirectoryData_Export MerchantMap : public std::map<Merchant::Key, Merchant>
	{
		public:
			static MerchantMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
			MerchantPtrMap search( const std::string& strSearch, bool bIncludeDeleted = false ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Merchant_h__ */
