#ifndef __DirectoryData_MerchantComment_h__
#define __DirectoryData_MerchantComment_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Base.h>
#include <DirectoryData/Merchant.h>

#include <Auth4cData/User.h>

#include <Base4c/Timestamp.h>

namespace DirectoryData
{
	class DirectoryData_Export MerchantComment : public Base
	{
		public:
			class DirectoryData_Export Key : public Merchant::Key
			{
				private:
					Base4c::Timestamp _tsCommentWhen;

				public:
					Key();
					Key( const Merchant::Key& kMerchant, const Base4c::Timestamp& tsCommentWhen );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Timestamp& commentWhen() const;

					std::string str() const;
			};

		private:
			Key _key;
			std::string _strLegend;
			int _nSeverity;
			Auth4cData::User _userCommentBy;

		public:
			MerchantComment();
			MerchantComment( const Key& key );
			MerchantComment( const MerchantComment& comment );
			virtual ~MerchantComment();

			MerchantComment& operator=( const MerchantComment& comment );
			virtual MerchantComment& merge( const MerchantComment& comment );

			const MerchantComment::Key& key() const;
			const Base4c::Timestamp& commentWhen() const;
			const std::string& legend() const;
			int severity() const;
			const Auth4cData::User& commentBy() const;

			void setLegend( const std::string& strLegend );
			void setSeverity( int nSeverity );

			static MerchantComment fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static MerchantComment create( const MerchantComment& comment ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};


	class DirectoryData_Export MerchantCommentMap : public std::map<MerchantComment::Key, MerchantComment>
	{
		public:
			static MerchantCommentMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
			MerchantCommentMap& operator=( const MerchantCommentMap& commentS );
	};
}

#endif /* !__DirectoryData_MerchantComment_h__ */
