#ifndef __DirectoryData_MerchantInvoice_h__
#define __DirectoryData_MerchantInvoice_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Base.h>

#include <DirectoryData/Merchant.h>

#include <Base4c/Timestamp.h>

#include <map>

namespace DirectoryData
{
	class Product;
	class InvoiceProduct;
	class InvoiceProductMap;

	class DirectoryData_Export MerchantInvoice : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					int _nInvoice;

				public:
					Key();
					Key( int nInvoice );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int invoice() const;

					virtual std::string str() const;
			};

			enum Status { stNew, stOpen, stClosed };

		private:
			Key _key;
			Merchant* _pmerchant;
			Base4c::Timestamp _tsCreatedWhen;
			Status _status;
			InvoiceProductMap* _pproductS;

		public:
			MerchantInvoice();
			MerchantInvoice( const Key& key );
			MerchantInvoice( const MerchantInvoice& invoice );
			virtual ~MerchantInvoice();

			MerchantInvoice& operator=( const MerchantInvoice& invoice );

			const Key& key() const;
			Merchant& merchant() const throw( DirectoryData::Exception );
			const Base4c::Timestamp& createdWhen() const;
			Status status() const;

			void setMerchant( Merchant& merchant );
			void setStatus( Status status );

			const InvoiceProductMap& products( bool bReload = false ) const throw( DirectoryData::Exception );
			InvoiceProductMap& products( bool bReload = false ) throw( DirectoryData::Exception );
			InvoiceProduct& addProduct( const Product& product, int nQuantity, double dPrice ) throw( DirectoryData::Exception );

			static MerchantInvoice fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static MerchantInvoice create( const MerchantInvoice& invoice ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

			static int nextInvoice( RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
	};

	class DirectoryData_Export MerchantInvoiceMap : public std::map<MerchantInvoice::Key, MerchantInvoice>
	{
		public:
			static MerchantInvoiceMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_MerchantInvoice_h__ */
