#ifndef __DirectoryData_Message_h__
#define __DirectoryData_Message_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Exception.h>
#include <DirectoryData/MessageStatus.h>
#include <DirectoryData/Sender.h>

#include <Base4c/Guid.h>
#include <Base4c/Timestamp.h>

#include <map>

namespace DirectoryData
{
	class Mailer;
	class MessageRecipientMap;

	class DirectoryData_Export Message : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Base4c::Guid _guidMessage;

				public:
					Key();
					Key( const Base4c::Guid& guidMessage );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Guid& message() const;

					std::string str() const;
			};

		private:
			Key _key;
			const MessageStatus* _pmessageStatus;
			Base4c::Timestamp _tsUpdatedWhen;
			const Sender* _psender;
			std::string _strSubject;
			std::string _strLegend;
			MessageRecipientMap* _precipientS;

		public:
			Message();
			Message( const Message::Key& key );
			Message( const Message& message );
			virtual ~Message();

			Message& operator=( const Message& message );

			const Message::Key& key() const;
			const MessageStatus& messageStatus() const throw( DirectoryData::Exception );
			const Base4c::Timestamp& updatedWhen() const;
			const Sender& sender() const throw( DirectoryData::Exception );
			const std::string& subject() const;
			const std::string& legend() const;
			MessageRecipientMap& recipients() const;

			void setSender( const Sender& sender );
			void setSubject( const std::string& strSubject );
			void setLegend( const std::string& strLegend );

			static Message fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );

		protected:
			static Message create( const Message& message ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

			void loadRecipients( RfcDal::DalDatabase& db );
			void saveRecipients( RfcDal::DalDatabase& db );

			friend class Mailer;
	};

	class MessageMap : public std::map<Message::Key, Message>
	{
		public:
			static MessageMap fromResult( RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db ) throw( DirectoryData::Exception );
	};

	class MessagePtrMap : public std::map<Message::Key, Message*>
	{
	};
}

#endif /* !__DirectoryData_Message_h__ */
