#ifndef __DirectoryData_MessageRecipient_h__
#define __DirectoryData_MessageRecipient_h__

#include <DirectoryData/Merchant.h>
#include <DirectoryData/Message.h>

namespace DirectoryData
{
	class Mailer;

	class DirectoryData_Export MessageRecipient : public Base
	{
		public:
			class DirectoryData_Export Key : public Message::Key, public Merchant::Key
			{
				public:
					Key();
					Key( const Message::Key& kMessage, const Merchant::Key& kMerchant );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					std::string str() const;
			};

		private:
			Key _key;
			bool _bError;
			std::string _strLegend;
			Merchant* _pmerchant;

		public:
			MessageRecipient();
			MessageRecipient( const MessageRecipient::Key& key );
			MessageRecipient( const Message& msg, Merchant& merchant );
			MessageRecipient( const MessageRecipient& recipient );
			virtual ~MessageRecipient();

			MessageRecipient& operator=( const MessageRecipient& recipient );

			const Key& key() const;
			bool error() const;
			const std::string& legend() const;
			Merchant& merchant() const throw( DirectoryData::Exception );

			static MessageRecipient fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );

		protected:
			static MessageRecipient create( const MessageRecipient& recipient ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

			friend class Mailer;
			friend class Message;
	};

	class DirectoryData_Export MessageRecipientMap : public std::map<MessageRecipient::Key, MessageRecipient>
	{
		public:
			static MessageRecipientMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
//			MessageRecipientMap& operator=( const MessageRecipientMap bookingS );
	};
}

#endif /* !__DirectoryData_MessageRecipient_h__ */
