#ifndef __DirectoryData_Pitch_h__
#define __DirectoryData_Pitch_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/PitchType.h>
#include <DirectoryData/Address.h>

#include <Base4c/Guid.h>
#include <Base4c/Date.h>

#include <librfcdal/SqlGSPoint.h>
#include <librfcdal/DalDatabase.h>

#include <map>

namespace DirectoryData
{
	class PitchBooking;
	class PitchBookingPtrMap;
	class DirectoryFactory;
	class FestivalBooking;

	class DirectoryData_Export Pitch : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					Base4c::Guid _guidPitch;

				public:
					Key();
					Key( const Base4c::Guid& guidPitch );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Guid& pitch() const;

					std::string str() const;
			};

		private:
			Key _key;
			const PitchType* _ptype;
			std::string _strName;
			Address* _paddress;
			RfcDal::SqlGSPoint _location;
			Base4c::Date _dtRetired;
			PitchBookingPtrMap* _ppitchBookingS;

		public:
			Pitch();
			Pitch( const Key& key );
			Pitch( const Pitch& pitch );
			virtual ~Pitch();

			Pitch& operator=( const Pitch& pitch );
			Pitch& merge( const Pitch& pitch );

			const Key& key() const;
			const PitchType& pitchType() const throw( DirectoryData::Exception );
			const std::string& name() const;
			const Address& address() const throw( DirectoryData::Exception );
			const RfcDal::SqlGSPoint& location() const;
			const Base4c::Date& retired() const;
			bool isRetired() const;
			const PitchBookingPtrMap& pitchBookings() const;

			void setPitchType( const PitchType& type );
			void setName( const std::string& strName );
			void setAddress( Address& address );
			void setLocation( const RfcDal::SqlGSPoint& location );
			void setRetired( const Base4c::Date& dtRetired );
			PitchBookingPtrMap& pitchBookings();

			Address& address() throw( DirectoryData::Exception );

			static Pitch fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Pitch create( const Pitch& pitch ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );

		protected:
			void addPitchBooking( PitchBooking& booking ) throw( DirectoryData::Exception );
			void removePitchBooking( const PitchBooking& booking ) throw( DirectoryData::Exception );

			friend class DirectoryFactory;
			friend class FestivalBooking;
	};

	class DirectoryData_Export PitchMap : public std::map<Pitch::Key, Pitch>
	{
		public:
			static PitchMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};

	class DirectoryData_Export PitchPtrMap : public std::map<Pitch::Key, Pitch*>
	{
	};
}

#endif /* !__DirectoryData_Pitch_h__ */
