#ifndef __DirectoryData_PitchBooking_h__
#define __DirectoryData_PitchBooking_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/FestivalBooking.h>
#include <DirectoryData/Pitch.h>

namespace DirectoryData
{
	class DirectoryData_Export PitchBooking : public Base
	{
		public:
			class DirectoryData_Export Key : public FestivalBooking::Key, public Pitch::Key
			{
				public:
					Key();
					Key( const FestivalBooking::Key& kBooking, const Pitch::Key& kPitch );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					std::string str() const;
			};

		private:
			Key _key;
			FestivalBooking* _pfestivalBooking;
			Pitch* _ppitch;

		public:
			PitchBooking();
			PitchBooking( const Key& key );
			PitchBooking( const Key& key, FestivalBooking& festivalBooking, Pitch& pitch );
			PitchBooking( const PitchBooking& booking );
			virtual ~PitchBooking();

			PitchBooking& operator=( const PitchBooking& booking );

			const Key& key() const;

			FestivalBooking& festivalBooking() const throw( DirectoryData::Exception );
			Pitch& pitch() const throw( DirectoryData::Exception );

			static PitchBooking fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static PitchBooking create( const PitchBooking& booking ) throw( DirectoryData::Exception );
			void remove() const throw( DirectoryData::Exception );
	};

	class DirectoryData_Export PitchBookingMap : public std::map<PitchBooking::Key, PitchBooking>
	{
		public:
			static PitchBookingMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
			PitchBookingMap& operator=( const PitchBookingMap bookingS );
	};

	class DirectoryData_Export PitchBookingPtrMap : public std::map<PitchBooking::Key, PitchBooking*>
	{
	};
}

#endif /* !__DirectoryData_PitchBooking_h__ */
