#ifndef __DirectoryData_Product_h__
#define __DirectoryData_Product_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Base.h>
#include <DirectoryData/Exception.h>
#include <DirectoryData/MerchantType.h>

#include <Base4c/Date.h>

#include <librfcdal/DalDatabase.h>

namespace DirectoryData
{
	class ProductPrice;
	class ProductPriceMap;

	class DirectoryData_Export Product : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					int _nProduct;

				public:
					Key();
					Key( int nProduct );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					int product() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strDescription;
			int _nQtyLimit;
			ProductPriceMap* _ppriceS;

		public:
			Product();
			Product( const Key& key );
			Product( const Product& product );
			virtual ~Product();

			Product& operator=( const Product& product );

			const Key& key() const;
			const std::string& description() const;
			int qtyLimit() const;

			void setDescription( const std::string& strDescription );
			void setQtyLimit( int nQtyLimit );

			const ProductPriceMap& productPrices( bool bReload = false ) const throw( DirectoryData::Exception );
			ProductPriceMap& productPrices( bool bReload = false ) throw( DirectoryData::Exception );
			ProductPrice& addProductPrice( const Base4c::Date& dtValidFrom, const Base4c::Date& dtValidUntil, const MerchantType& type, double dPrice ) throw( DirectoryData::Exception );

			static Product fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Product create( const Product& product ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export ProductMap : public std::map<Product::Key, Product>
	{
		public:
			static ProductMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Product_h__ */
