#ifndef __DirectoryData_ProductPrice_h__
#define __DirectoryData_ProductPrice_h__

#include <DirectoryData/DirectoryData.h>

#include <DirectoryData/Product.h>
#include <DirectoryData/MerchantType.h>

#include <Base4c/Date.h>

#include <map>

namespace DirectoryData
{
	class DirectoryData_Export ProductPrice : public Base
	{
		public:
			class DirectoryData_Export Key : public Product::Key, public MerchantType::Key
			{
				private:
					Base4c::Date _dtValidFrom;

				public:
					Key();
					Key( const Product::Key& kProduct, const Base4c::Date& dtValidFrom, const MerchantType::Key& kType );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Date& validFrom() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			Base4c::Date _dtValidUntil;
			double _dPrice;

		public:
			ProductPrice();
			ProductPrice( const Key& key );
			ProductPrice( const ProductPrice& prodprice );
			virtual ~ProductPrice();

			ProductPrice& operator=( const ProductPrice& prodprice );

			const Key& key() const;
			const Base4c::Date& validUntil() const;
			double price() const;

			DirectoryData::Product& product() const throw( DirectoryData::Exception );
			DirectoryData::MerchantType& merchantType() const throw( DirectoryData::Exception );

			void setValidUntil( const Base4c::Date& dtValidUntil );
			void setPrice( double dPrice );

			static ProductPrice fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static ProductPrice create( const ProductPrice& prodprice ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export ProductPriceMap : public std::map<ProductPrice::Key, ProductPrice>
	{
		public:
			static ProductPriceMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_ProductPrice_h__ */
