#ifndef __DirectoryData_Sender_h__
#define __DirectoryData_Sender_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Exception.h>

#include <librfcdal/DalDatabase.h>

namespace DirectoryData
{
	class DirectoryData_Export Sender : public Base
	{
		public:
			class DirectoryData_Export Key : public Base::Key
			{
				private:
					std::string _strSender;

				public:
					Key();
					Key( const std::string& strEnumerator );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& sender() const;

					virtual std::string str() const;
			};

		private:
			Key _key;
			std::string _strDescription;
			std::string _strSignature;

		public:
			Sender();
			Sender( const Sender::Key& key );
			Sender( const Sender& sender );
			virtual ~Sender();

			Sender& operator=( const Sender& sender );

			const Sender::Key& key() const;
			const std::string& description() const;
			const std::string& signature() const;

			void setDescription( const std::string& strDescription );
			void setSignature( const std::string& strSignature );

			static Sender fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Sender create( const Sender& sender ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export SenderMap : public std::map<Sender::Key, Sender>
	{
		public:
			static SenderMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Sender_h__ */
