#ifndef __DirectoryData_Sponsor_h__
#define __DirectoryData_Sponsor_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Festival.h>
#include <DirectoryData/Merchant.h>

#include <map>

namespace DirectoryData
{
	class DirectoryData_Export Sponsor : public Base
	{
		public:
			class DirectoryData_Export Key : public Festival::Key, public Merchant::Key
			{
				public:
					Key();
					Key( const Festival::Key& kFestival, const Merchant::Key& kMerchant );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					std::string str() const;
			};

		private:
			Key _key;
			int _nSponsorSeq;
			Festival* _pfestival;
			Merchant* _pmerchant;

		public:
			Sponsor();
			Sponsor( Festival& festival, Merchant& merchant );
			Sponsor( const Sponsor& booking );
			virtual ~Sponsor();

			Sponsor operator=( const Sponsor& booking );

			const Key& key() const;
			int sponsorSeq() const;

			Festival& festival() const throw( DirectoryData::Exception );
			Merchant& merchant() const throw( DirectoryData::Exception );

			void setSponsorSeq( int nSeq );

			static Sponsor fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Sponsor create( const Sponsor& booking ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export SponsorMap : public std::map<Sponsor::Key, Sponsor>
	{
		public:
			static SponsorMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
			SponsorMap& operator=( const SponsorMap bookingS );
	};

	class DirectoryData_Export SponsorPtrMap : public std::map<Sponsor::Key, Sponsor*>
	{
	};
}

#endif /* !__DirectoryData_Sponsor_h__ */
