#ifndef __DirectoryData_Stage_h__
#define __DirectoryData_Stage_h__

#include <DirectoryData/Enumerator.h>
#include <DirectoryData/Exception.h>

#include <librfcdal/DalDatabase.h>

#include <map>

namespace DirectoryData
{
	class StageBooking;
	class StageBookingMap;

	class DirectoryData_Export Stage : public Enumerator
	{
		private:
			int _nSeq;
			std::string _strNode;
			StageBookingMap* _pstageBookingS;

		public:
			Stage();
			Stage( const Key& key );
			Stage( const Stage& stage );
			virtual ~Stage();

			Stage& operator=( const Stage& stage );
			Stage& merge( const Stage& stage );

			int seq() const;
			const std::string& node() const;
			const StageBookingMap& stageBookings() const;

			void setSeq( int nSeq );
			void setNode( const std::string& strNode );
			StageBookingMap& stageBookings();

			StageBooking& addStageBooking( StageBooking& booking ) throw( DirectoryData::Exception );
			void removeStageBooking( const StageBooking& booking ) throw( DirectoryData::Exception );

			static Stage fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static Stage create( const Stage& stage ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
	};

	class DirectoryData_Export StageMap : public std::map<Stage::Key, Stage>
	{
		public:
			static StageMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_Stage_h__ */
