#ifndef __DirectoryData_StageBooking_h__
#define __DirectoryData_StageBooking_h__

#include <DirectoryData/Base.h>
#include <DirectoryData/Festival.h>
#include <DirectoryData/Stage.h>
#include <DirectoryData/Exception.h>

#include <librfcdal/DalDatabase.h>

#include <Base4c/Time.h>

#include <map>

namespace DirectoryData
{
	class Act;

	class DirectoryData_Export StageBooking : public Base
	{
		public:
			class DirectoryData_Export Key : public Festival::Key, public Stage::Key
			{
				private:
					Base4c::Time _tmUpWhen;

				public:
					Key();
					Key( const Festival::Key& kFestival, const Stage::Key& kStage, const Base4c::Time& tmUpWhen );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const Base4c::Time& upWhen() const;

					std::string str() const;
			};

		private:
			Key _key;
			Act* _pact;
			Festival* _pfestival;
			Stage* _pstage;

		public:
			StageBooking();
//			StageBooking( const Key& key );
			StageBooking( Festival& festival, Stage& stage, const Base4c::Time& tmUpWhen );
			StageBooking( const StageBooking& booking );
			virtual ~StageBooking();

			StageBooking& operator=( const StageBooking& booking );
			StageBooking& merge( const StageBooking& booking );

			const Key& key() const;
			Act& act() const throw( DirectoryData::Exception );

			void setAct( Act& act );

			Festival& festival() const throw( DirectoryData::Exception );
			Stage& stage() const throw( DirectoryData::Exception );

			static StageBooking fromRow( const RfcDal::DalDatabase::Row& row ) throw( DirectoryData::Exception );
			static StageBooking create( const StageBooking& booking ) throw( DirectoryData::Exception );
			void update() throw( DirectoryData::Exception );
			void remove() const throw( DirectoryData::Exception );
	};

	class DirectoryData_Export StageBookingMap : public std::map<StageBooking::Key, StageBooking>
	{
		public:
			static StageBookingMap fromResult( const RfcDal::DalDatabase::Result& result ) throw( DirectoryData::Exception );
	};
}

#endif /* !__DirectoryData_StageBooking_h__ */
